"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module UsageLogging
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrontendFeatureUsageTelemetryClient = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const telemetry_client_1 = require("@bentley/telemetry-client");
const UsageLoggingClient_1 = require("./UsageLoggingClient");
const UsageLoggingClientLoggerCategories_1 = require("./UsageLoggingClientLoggerCategories");
/**
 * @internal
 * Wrapper around [[UsageLoggingClient]] that implements [[TelemetryClient]]
 */
class FrontendFeatureUsageTelemetryClient extends telemetry_client_1.FrontendTelemetryClient {
    constructor(_hostname = "IMJSDEFAULT") {
        super();
        this._hostname = _hostname;
        this._usageLoggingClient = new UsageLoggingClient_1.UsageLoggingClient();
    }
    async _postTelemetry(requestContext, clientTelemetryEvent) {
        const featureLogEntry = FrontendFeatureUsageTelemetryClient.getFeatureLogEntry(clientTelemetryEvent, this._hostname);
        await this._usageLoggingClient.logFeatureUsage(requestContext, featureLogEntry);
    }
    static getFeatureLogEntry(clientTelemetryEvent, hostname) {
        if (!clientTelemetryEvent.eventId || !bentleyjs_core_1.Guid.isGuid(clientTelemetryEvent.eventId)) { // valid featureId Guid is required to post to ULAS
            const message = `Could not log feature usage with name: ${clientTelemetryEvent.eventName}. Valid featureId Guid is required by ULAS. Supplied featureId: ${clientTelemetryEvent.eventId}`;
            bentleyjs_core_1.Logger.logError(UsageLoggingClientLoggerCategories_1.UsageLoggingClientLoggerCategory.Telemetry, message, () => ({ clientTelemetryEvent }));
            throw new Error(message);
        }
        const featureLogEntry = new UsageLoggingClient_1.FeatureLogEntry(clientTelemetryEvent.eventId, hostname, UsageLoggingClient_1.UsageType.Production, clientTelemetryEvent.contextId);
        if (clientTelemetryEvent.time) {
            featureLogEntry.startTime = clientTelemetryEvent.time.startTime;
            featureLogEntry.endTime = clientTelemetryEvent.time.endTime;
        }
        const eventProps = clientTelemetryEvent.getProperties();
        // remove redundant fields
        delete eventProps.eventId;
        delete eventProps.contextId;
        delete eventProps.time;
        const eventStringProps = {};
        for (const propName in eventProps) { // eslint-disable-line guard-for-in
            eventStringProps[propName] = JSON.stringify(propName);
        }
        featureLogEntry.additionalData = eventStringProps;
        return featureLogEntry;
    }
}
exports.FrontendFeatureUsageTelemetryClient = FrontendFeatureUsageTelemetryClient;
//# sourceMappingURL=FrontendFeatureUsageTelemetryClient.js.map