"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogEntryConverter = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module UsageLogging
 */
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const UsageLoggingClient_1 = require("./UsageLoggingClient");
const UsageLoggingClientLoggerCategories_1 = require("./UsageLoggingClientLoggerCategories");
const loggerCategory = UsageLoggingClientLoggerCategories_1.UsageLoggingClientLoggerCategory.Client;
/** @internal */
class LogEntryConverter {
    /**
     * Extracts the application version from the supplied request context
     * @param requestContext The client request context
     * @returns The application version for the request context
     */
    static getApplicationVersion(requestContext) {
        const applicationVersion = requestContext.applicationVersion;
        const defaultVersion = { major: 1, minor: 0 };
        if (!applicationVersion) {
            bentleyjs_core_1.Logger.logWarning(loggerCategory, "ApplicationVersion was not specified. Set up IModelApp.applicationVersion for frontend applications, or IModelHost.applicationVersion for agents", () => ({ applicationVersion }));
            return defaultVersion;
        }
        const versionSplit = applicationVersion.split(".");
        const length = versionSplit.length;
        if (length < 2) {
            bentleyjs_core_1.Logger.logWarning(loggerCategory, "ApplicationVersion is not valid", () => ({ applicationVersion }));
            return defaultVersion;
        }
        const major = parseInt(versionSplit[0], 10);
        if (typeof major === "undefined") {
            bentleyjs_core_1.Logger.logWarning(loggerCategory, "ApplicationVersion is not valid", () => ({ applicationVersion }));
            return defaultVersion;
        }
        const minor = parseInt(versionSplit[1], 10);
        if (typeof minor === "undefined") {
            bentleyjs_core_1.Logger.logWarning(loggerCategory, "ApplicationVersion is not valid", () => ({ applicationVersion }));
            return { major, minor: 0 };
        }
        let sub1;
        let sub2;
        if (length > 2) {
            sub1 = parseInt(versionSplit[2], 10);
            if (length > 3 && !isNaN(sub1)) {
                sub2 = parseInt(versionSplit[3], 10) || undefined;
            }
        }
        return { major, minor, sub1, sub2 };
    }
    /**
     * Extracts the application id from the supplied request context
     * @param requestContext The client request context
     * @returns The application id for the request context
     */
    static getApplicationId(requestContext) {
        const defaultId = 2686; // iModel.js
        if (!requestContext.applicationId) {
            bentleyjs_core_1.Logger.logWarning(loggerCategory, "ApplicationId was not specified. Set up IModelApp.applicationId for frontend applications, or IModelHost.applicationId for agents");
            return defaultId;
        }
        return parseInt(requestContext.applicationId, 10) || defaultId;
    }
    /**
     * Extracts the session id from the supplied request context
     * @param requestContext The client request context
     * @returns The session id for the request context
     */
    static getSessionId(requestContext) {
        if (!bentleyjs_core_1.Guid.isGuid(requestContext.sessionId)) {
            bentleyjs_core_1.Logger.logWarning(loggerCategory, "Specified sessionId is not a valid Guid. Set up IModelApp.sessionId for frontend applications, or IModelHost.sessionId for agents");
            return bentleyjs_core_1.Guid.empty;
        }
        return requestContext.sessionId;
    }
    /**
     * @internal
     * @param requestContext
     * @param entry
     */
    static toUsageLogJson(requestContext, entry) {
        const productId = LogEntryConverter.getApplicationId(requestContext);
        const productVersion = LogEntryConverter.getApplicationVersion(requestContext);
        const sessionId = LogEntryConverter.getSessionId(requestContext);
        const machineName = LogEntryConverter.prepareMachineName(entry.hostName);
        const versionNumber = LogEntryConverter.toVersionNumber(productVersion);
        const usageType = LogEntryConverter.usageTypeToString(entry.usageType);
        return {
            hID: machineName,
            polID: LogEntryConverter._policyFileId,
            secID: LogEntryConverter._securableId,
            prdid: productId,
            fstr: LogEntryConverter._featureString,
            ver: versionNumber,
            projID: entry.contextId,
            corID: sessionId,
            lVer: LogEntryConverter._logEntryVersion,
            lSrc: LogEntryConverter._logPostingSource,
            uType: usageType,
        };
    }
    /**
     * @internal
     * @param requestContext
     * @param entry
     */
    static toFeatureLogJson(requestContext, entry) {
        var _a, _b, _c;
        const productId = LogEntryConverter.getApplicationId(requestContext);
        const productVersion = LogEntryConverter.getApplicationVersion(requestContext);
        const sessionId = LogEntryConverter.getSessionId(requestContext);
        const versionNumber = LogEntryConverter.toVersionNumber(productVersion);
        const machineName = LogEntryConverter.prepareMachineName(entry.hostName);
        const usageType = LogEntryConverter.usageTypeToString(entry.usageType);
        const startDateZ = (_a = entry.startTime) === null || _a === void 0 ? void 0 : _a.toISOString();
        const endDateZ = (_b = entry.endTime) === null || _b === void 0 ? void 0 : _b.toISOString();
        const featureMetaData = [];
        for (const att in entry.additionalData) { // eslint-disable-line guard-for-in
            featureMetaData.push({ name: att, value: entry.additionalData[att] });
        }
        const contextId = (_c = entry.contextId) !== null && _c !== void 0 ? _c : "99999999-9999-9999-9999-999999999999"; // All iTwin applications are obligated by ULAS to send the 9-Guid (instead of the 0-Guid or undefined) to denote a global project scope.
        const entryJson = {
            hID: machineName,
            polID: LogEntryConverter._policyFileId,
            secID: LogEntryConverter._securableId,
            prdid: productId,
            fstr: LogEntryConverter._featureString,
            ver: versionNumber,
            projID: contextId,
            corID: sessionId,
            lVer: LogEntryConverter._logEntryVersion,
            lSrc: LogEntryConverter._logPostingSource,
            uType: usageType,
            ftrID: entry.featureId,
            sDateZ: startDateZ,
            eDateZ: endDateZ,
            uData: featureMetaData,
        };
        return entryJson;
    }
    static toVersionNumber(version) {
        // version must be encoded into a single number where each version digit is padded out to 4 digits
        // and the version is always considered to have 4 digits.
        // Ex: 3.99.4 -> 3.99.4.0 -> 3009900040000
        let verNumber = !!version.sub2 ? version.sub2 : 0;
        verNumber += 10000 * (!!version.sub1 ? version.sub1 : 0);
        verNumber += Math.pow(10000, 2) * version.minor;
        verNumber += Math.pow(10000, 3) * version.major;
        return verNumber;
    }
    static prepareMachineName(machineName) {
        if (!machineName || machineName.length === 0)
            return "";
        if (machineName === "::1" || machineName === "127.0.0.1")
            return "localhost";
        return machineName.toLowerCase();
    }
    /* private static prepareUserName(userName: string, machineName: string): string {
      if (!userName || userName.length === 0)
        return "";
  
      let preparedUserName: string = userName;
  
      const backslashPos: number = userName.indexOf("\\");
      if (backslashPos >= 0)
        preparedUserName = userName.substr(backslashPos + 1);
      else {
        const slashPos: number = userName.indexOf("/");
        if (slashPos >= 0)
          preparedUserName = userName.substr(slashPos + 1);
      }
  
      preparedUserName = preparedUserName.toLowerCase();
      if (!!machineName && machineName.length > 0 && (preparedUserName.includes("administrator") || preparedUserName.includes("system")))
        preparedUserName = `${machineName.toLowerCase()}\\${preparedUserName}`;
  
      return preparedUserName;
    } */
    static usageTypeToString(val) {
        switch (val) {
            case UsageLoggingClient_1.UsageType.Beta:
                return "Beta";
            case UsageLoggingClient_1.UsageType.HomeUse:
                return "HomeUse";
            case UsageLoggingClient_1.UsageType.PreActivation:
                return "PreActivation";
            case UsageLoggingClient_1.UsageType.Production:
                return "Production";
            case UsageLoggingClient_1.UsageType.Trial:
                return "Trial";
            default:
                throw new Error("Unhandled UsageType enum value");
        }
    }
}
exports.LogEntryConverter = LogEntryConverter;
// for now this is always 1
LogEntryConverter._logEntryVersion = 1;
// this is a real-time client, i.e. it sends the requests right away without caching or aggregating.
LogEntryConverter._logPostingSource = "RealTime";
// fStr argument is empty for now
LogEntryConverter._featureString = "";
LogEntryConverter._policyFileId = bentleyjs_core_1.Guid.createValue();
LogEntryConverter._securableId = bentleyjs_core_1.Guid.createValue();
//# sourceMappingURL=LogEntryConverter.js.map