/** @packageDocumentation
 * @module UsageLogging
 */
import { BentleyStatus, GuidString } from "@bentley/bentleyjs-core";
import { AuthorizedClientRequestContext, Client, RequestOptions } from "@bentley/itwin-client";
/** Represents one of the potential usage types.
 * See also
 *  - [[UsageLogEntry]], [[FeatureLogEntry]]
 *  - *UsageType* entry on [ULAS Swagger](https://connect-ulastm.bentley.com/Bentley.ULAS.SwaggerUI/SwaggerWebApp/?urls.primaryName=ULAS%20Posting%20Service%20v1)
 *  site (section *Models*)
 * @internal
 */
export declare enum UsageType {
    Production = 0,
    Trial = 1,
    Beta = 2,
    HomeUse = 3,
    PreActivation = 4
}
/** Represents the version of the product logging usage or features.
 * See also [[UsageLogEntry]], [[FeatureLogEntry]].
 * @internal
 */
export interface ProductVersion {
    major: number;
    minor: number;
    sub1?: number;
    sub2?: number;
}
/**
 * Usage log entry data that is submitted to the ULAS Posting Service.
 * See also
 *  - [[UlasClient]]
 *  - *UsageLogEntry* entry on [ULAS Swagger](https://connect-ulastm.bentley.com/Bentley.ULAS.SwaggerUI/SwaggerWebApp/index.html?urls.primaryName=ULAS%20Posting%20Service%20v1)
 *  site (section *Models*)
 * @internal
 */
export declare class UsageLogEntry {
    /** The GUID of the context that the usage should be associated with. */
    contextId?: GuidString;
    /** Name of the client machine from which usage is logged */
    readonly hostName: string;
    /** The type of usage that occurred on the client. It is acting as a filter to eliminate records from log processing that
     * should not count towards a customer’s peak processing.
     */
    readonly usageType: UsageType;
    /** Timestamp against which the usage is logged.
     * It is set at construction time of this object.
     */
    readonly timestamp: string;
    /** Creates a new UsageLogEntry object.
     *  This also sets the timestamp against which the usage will be logged.
     *  @param hostName Name of the client machine from which usage is logged.
     *  @param usageType Usage type (see [[UsageType]])
     *  @param contextId The GUID of the context that the usage should be associated with.
     */
    constructor(hostName: string, usageType: UsageType, contextId?: GuidString);
}
/**
 * Standard feature log entry data that is submitted to the ULAS Posting Service.
 * See also
 *  - [[UlasClient]]
 *  - *FeatureLogEntry* entry on [ULAS Swagger](https://connect-ulastm.bentley.com/Bentley.ULAS.SwaggerUI/SwaggerWebApp/index.html?urls.primaryName=ULAS%20Posting%20Service%20v1)
 *  site (section *Models*)
 * @internal
 */
export declare class FeatureLogEntry {
    /** ID of the feature to log (from the Global Feature Registry). */
    readonly featureId: GuidString;
    /** Name of the client machine from which usage is logged. */
    readonly hostName: string;
    /** The type of usage that occurred on the client. It is acting as a filter to eliminate records from log processing that
     * should not count towards a customer’s peak processing.
     */
    readonly usageType: UsageType;
    /** The GUID of the context that the usage should be associated with. */
    readonly contextId?: string | undefined;
    /** Additional user-defined metadata for the feature usage. */
    additionalData: {
        [key: string]: string;
    };
    correlationId: string;
    /**
     * Time at which feature usage began
     * Defaults to current time
     */
    startTime: Date;
    /**
     * Time at which feature usage ended
     * Defaults to current time
     */
    endTime: Date;
    /** Creates a new FeatureLogEntry object.
     *  This also sets the timestamp against which the feature will be logged.
     *  @param featureId Feature ID from the Global Feature Registry which is being logged.
     *  @param hostName Name of the client machine from which the feature is being logged.
     *  @param usageType Usage type (see [[UsageType]])
     *  @param contextId The GUID of the context that the usage should be associated with.
     */
    constructor(
    /** ID of the feature to log (from the Global Feature Registry). */
    featureId: GuidString, 
    /** Name of the client machine from which usage is logged. */
    hostName: string, 
    /** The type of usage that occurred on the client. It is acting as a filter to eliminate records from log processing that
     * should not count towards a customer’s peak processing.
     */
    usageType: UsageType, 
    /** The GUID of the context that the usage should be associated with. */
    contextId?: string | undefined);
}
/**
 * Feature log entry data that can be independently submitted to the ULAS Posting Service to denote the start of a feature usage session.
 * @internal
 */
export declare class StartFeatureLogEntry extends FeatureLogEntry {
    readonly startTime: Date;
    readonly endTime: Date;
    /**
     * @param featureId
     * @param hostName
     * @param usageType
     * @param contextId
     * @param startTime Sets the startTime of the feature entry if provided, defaults to the current time when omitted.
     */
    constructor(featureId: GuidString, hostName: string, usageType: UsageType, contextId?: GuidString, startTime?: Date);
}
/**
 * Feature log entry data that can be independently submitted to the ULAS Posting Service to denote the end of a feature usage session.
 * Expected to always be paired with a corresponding StartFeatureLogEntry (related via correlationId)
 * @internal
 */
export declare class EndFeatureLogEntry extends FeatureLogEntry {
    readonly startTime: Date;
    readonly endTime: Date;
    private constructor();
    /**
     * Creates a feature log entry denoting the end of a usage session using a corresponding start entry.
     * @param startEntry
     * @param endTime Sets the end time of the feature entry if provided, defaults to the current time when omitted.
     */
    static createFromStartEntry(startEntry: StartFeatureLogEntry, endTime?: Date): EndFeatureLogEntry;
}
/**
 * Response from posting a [[UsageLogEntry]] or [[FeatureLogEntry]] with the [[UlasClient]].
 * See also *LogPostingResponse* entry on [ULAS Swagger](https://connect-ulastm.bentley.com/Bentley.ULAS.SwaggerUI/SwaggerWebApp/?urls.primaryName=ULAS%20Posting%20Service%20v1)
 * site (section *Models*)
 * @internal
 */
export interface LogPostingResponse {
    status: BentleyStatus;
    message: string;
    time: number;
    requestId: GuidString;
}
/**
 * Client for the Bentley Usage Logging & Analysis Services (ULAS).
 * See also the two `POST` requests on [ULAS Swagger](https://connect-ulastm.bentley.com/Bentley.ULAS.SwaggerUI/SwaggerWebApp/?urls.primaryName=ULAS%20Posting%20Service%20v1)
 * @internal
 */
export declare class UsageLoggingClient extends Client {
    private static readonly _buddiSearchKey;
    /** Creates an instance of UlasClient. */
    constructor();
    /**
     * Gets name/key to query the service URLs from the URL Discovery Service ("Buddi")
     * @returns Search key for the URL.
     */
    protected getUrlSearchKey(): string;
    protected setupOptionDefaults(options: RequestOptions): Promise<void>;
    /**
     * Logs one or more feature entries via the ULAS service.
     * @param requestContext The client request context.
     * @param featureEntries One or more feature log entries.
     * @returns Response from the service.
     */
    logFeatureUsage(requestContext: AuthorizedClientRequestContext, ...featureEntries: FeatureLogEntry[]): Promise<LogPostingResponse>;
    private logEntry;
}
//# sourceMappingURL=UsageLoggingClient.d.ts.map