"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UsageLoggingClient = exports.EndFeatureLogEntry = exports.StartFeatureLogEntry = exports.FeatureLogEntry = exports.UsageLogEntry = exports.UsageType = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module UsageLogging
 */
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const LogEntryConverter_1 = require("./LogEntryConverter");
const UsageLoggingClientLoggerCategories_1 = require("./UsageLoggingClientLoggerCategories");
const loggerCategory = UsageLoggingClientLoggerCategories_1.UsageLoggingClientLoggerCategory.Client;
/** Represents one of the potential usage types.
 * See also
 *  - [[UsageLogEntry]], [[FeatureLogEntry]]
 *  - *UsageType* entry on [ULAS Swagger](https://connect-ulastm.bentley.com/Bentley.ULAS.SwaggerUI/SwaggerWebApp/?urls.primaryName=ULAS%20Posting%20Service%20v1)
 *  site (section *Models*)
 * @internal
 */
var UsageType;
(function (UsageType) {
    UsageType[UsageType["Production"] = 0] = "Production";
    UsageType[UsageType["Trial"] = 1] = "Trial";
    UsageType[UsageType["Beta"] = 2] = "Beta";
    UsageType[UsageType["HomeUse"] = 3] = "HomeUse";
    UsageType[UsageType["PreActivation"] = 4] = "PreActivation";
})(UsageType = exports.UsageType || (exports.UsageType = {}));
/**
 * Usage log entry data that is submitted to the ULAS Posting Service.
 * See also
 *  - [[UlasClient]]
 *  - *UsageLogEntry* entry on [ULAS Swagger](https://connect-ulastm.bentley.com/Bentley.ULAS.SwaggerUI/SwaggerWebApp/index.html?urls.primaryName=ULAS%20Posting%20Service%20v1)
 *  site (section *Models*)
 * @internal
 */
class UsageLogEntry {
    /** Creates a new UsageLogEntry object.
     *  This also sets the timestamp against which the usage will be logged.
     *  @param hostName Name of the client machine from which usage is logged.
     *  @param usageType Usage type (see [[UsageType]])
     *  @param contextId The GUID of the context that the usage should be associated with.
     */
    constructor(hostName, usageType, contextId) {
        this.hostName = hostName;
        this.usageType = usageType;
        this.contextId = contextId;
        this.timestamp = new Date().toISOString();
    }
}
exports.UsageLogEntry = UsageLogEntry;
/**
 * Standard feature log entry data that is submitted to the ULAS Posting Service.
 * See also
 *  - [[UlasClient]]
 *  - *FeatureLogEntry* entry on [ULAS Swagger](https://connect-ulastm.bentley.com/Bentley.ULAS.SwaggerUI/SwaggerWebApp/index.html?urls.primaryName=ULAS%20Posting%20Service%20v1)
 *  site (section *Models*)
 * @internal
 */
class FeatureLogEntry {
    /** Creates a new FeatureLogEntry object.
     *  This also sets the timestamp against which the feature will be logged.
     *  @param featureId Feature ID from the Global Feature Registry which is being logged.
     *  @param hostName Name of the client machine from which the feature is being logged.
     *  @param usageType Usage type (see [[UsageType]])
     *  @param contextId The GUID of the context that the usage should be associated with.
     */
    constructor(
    /** ID of the feature to log (from the Global Feature Registry). */
    featureId, 
    /** Name of the client machine from which usage is logged. */
    hostName, 
    /** The type of usage that occurred on the client. It is acting as a filter to eliminate records from log processing that
     * should not count towards a customer’s peak processing.
     */
    usageType, 
    /** The GUID of the context that the usage should be associated with. */
    contextId) {
        this.featureId = featureId;
        this.hostName = hostName;
        this.usageType = usageType;
        this.contextId = contextId;
        /** Additional user-defined metadata for the feature usage. */
        this.additionalData = {};
        this.correlationId = bentleyjs_core_1.Guid.createValue();
        /**
         * Time at which feature usage began
         * Defaults to current time
         */
        this.startTime = new Date();
        /**
         * Time at which feature usage ended
         * Defaults to current time
         */
        this.endTime = new Date();
    }
}
exports.FeatureLogEntry = FeatureLogEntry;
/**
 * Feature log entry data that can be independently submitted to the ULAS Posting Service to denote the start of a feature usage session.
 * @internal
 */
class StartFeatureLogEntry extends FeatureLogEntry {
    /**
     * @param featureId
     * @param hostName
     * @param usageType
     * @param contextId
     * @param startTime Sets the startTime of the feature entry if provided, defaults to the current time when omitted.
     */
    constructor(featureId, hostName, usageType, contextId, startTime = new Date()) {
        super(featureId, hostName, usageType, contextId);
        this.endTime = new Date("0001-01-01T00:00:00Z"); // ULAS spec dictates that start feature usage entries use the default date for endDate
        this.startTime = startTime;
    }
}
exports.StartFeatureLogEntry = StartFeatureLogEntry;
/**
 * Feature log entry data that can be independently submitted to the ULAS Posting Service to denote the end of a feature usage session.
 * Expected to always be paired with a corresponding StartFeatureLogEntry (related via correlationId)
 * @internal
 */
class EndFeatureLogEntry extends FeatureLogEntry {
    constructor(featureId, hostName, usageType, contextId, endTime = new Date()) {
        super(featureId, hostName, usageType, contextId);
        this.startTime = new Date("0001-01-01T00:00:00Z"); // ULAS spec dictates that end feature usage entries use the default date for startDate
        this.endTime = endTime;
    }
    /**
     * Creates a feature log entry denoting the end of a usage session using a corresponding start entry.
     * @param startEntry
     * @param endTime Sets the end time of the feature entry if provided, defaults to the current time when omitted.
     */
    static createFromStartEntry(startEntry, endTime) {
        const endEntry = new EndFeatureLogEntry(startEntry.featureId, startEntry.hostName, startEntry.usageType, startEntry.contextId, endTime);
        endEntry.correlationId = startEntry.correlationId;
        return endEntry;
    }
}
exports.EndFeatureLogEntry = EndFeatureLogEntry;
/**
 * Client for the Bentley Usage Logging & Analysis Services (ULAS).
 * See also the two `POST` requests on [ULAS Swagger](https://connect-ulastm.bentley.com/Bentley.ULAS.SwaggerUI/SwaggerWebApp/?urls.primaryName=ULAS%20Posting%20Service%20v1)
 * @internal
 */
class UsageLoggingClient extends itwin_client_1.Client {
    /** Creates an instance of UlasClient. */
    constructor() { super(); }
    /**
     * Gets name/key to query the service URLs from the URL Discovery Service ("Buddi")
     * @returns Search key for the URL.
     */
    getUrlSearchKey() { return UsageLoggingClient._buddiSearchKey; }
    async setupOptionDefaults(options) {
        await super.setupOptionDefaults(options);
        options.useCorsProxy = true;
    }
    /**
     * Logs one or more feature entries via the ULAS service.
     * @param requestContext The client request context.
     * @param featureEntries One or more feature log entries.
     * @returns Response from the service.
     */
    async logFeatureUsage(requestContext, ...featureEntries) {
        requestContext.enter();
        const featureLogEntryJson = []; // ULAS spec always expects an array of feature logs
        for (const entry of featureEntries) {
            const entryJson = LogEntryConverter_1.LogEntryConverter.toFeatureLogJson(requestContext, entry);
            featureLogEntryJson.push(entryJson);
        }
        const baseUrl = await this.getUrl(requestContext);
        requestContext.enter();
        const featurePostUrl = `${baseUrl}/featureLog`;
        return this.logEntry(requestContext, featurePostUrl, featureLogEntryJson);
    }
    async logEntry(requestContext, postUrl, jsonBody) {
        const token = requestContext.accessToken;
        const authString = token.toTokenString();
        const options = {
            method: "POST",
            headers: { authorization: authString },
            body: jsonBody,
        };
        await this.setupOptionDefaults(options);
        requestContext.enter();
        if (bentleyjs_core_1.Logger.isEnabled(loggerCategory, bentleyjs_core_1.LogLevel.Trace))
            bentleyjs_core_1.Logger.logTrace(loggerCategory, `Sending Usage Log REST request...`, () => ({ url: postUrl, body: jsonBody }));
        const resp = await itwin_client_1.request(requestContext, postUrl, options);
        requestContext.enter();
        const requestDetails = { url: postUrl, body: jsonBody, response: resp };
        if (bentleyjs_core_1.Logger.isEnabled(loggerCategory, bentleyjs_core_1.LogLevel.Trace))
            bentleyjs_core_1.Logger.logTrace(loggerCategory, `Sent Usage Log REST request.`, () => requestDetails);
        const respBody = resp.body;
        if (!respBody || !respBody.status || respBody.status.toLowerCase() !== "success")
            throw new Error(`Post Usage Log REST request failed ${!!respBody.msg ? `: ${respBody.msg}` : ""}. Details: ${JSON.stringify(requestDetails)}`);
        return { status: bentleyjs_core_1.BentleyStatus.SUCCESS, message: !!respBody.msg ? respBody.msg : "", time: !!respBody.time ? respBody.time : -1, requestId: respBody.reqID };
    }
}
exports.UsageLoggingClient = UsageLoggingClient;
UsageLoggingClient._buddiSearchKey = "UsageLoggingServices.RealtimeLogging.Url";
//# sourceMappingURL=UsageLoggingClient.js.map